@@ -228,6 +228,8 @@ int command(client_conn_t *conn, int *virus)
 	    thrmgr_setactivetask(NULL, "MULTISCAN");
 	    type = TYPE_MULTISCAN;
 	    scandata.group = group = thrmgr_group_new();
+	    if (!group)
+		return CL_EMEM;
 	    break;
 	case COMMAND_MULTISCANFILE:
 	    thrmgr_setactivetask(NULL, "MULTISCANFILE");
@@ -286,6 +288,7 @@ int command(client_conn_t *conn, int *virus)
 	    }
 	    return 0;
 	case COMMAND_INSTREAMSCAN:
+	    thrmgr_setactivetask(NULL, "INSTREAM");
 	    ret = scanfd(conn->scanfd, conn, NULL, engine, options, opts, desc, 1);
 	    if (ret == CL_VIRUS) {
 		*virus = 1;
