@@ -358,29 +358,29 @@ static int ea05(int desc, cli_ctx *ctx, char *tmpd) {
     if((i = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) {
       cli_dbgmsg("autoit: Can't create file %s\n", tempfile);
       free(UNP.outputbuf);
-      return CL_EIO;
+      return CL_ECREAT;
     }
     if(cli_writen(i, UNP.outputbuf, UNP.usize) != (int32_t)UNP.usize) {
       cli_dbgmsg("autoit: cannot write %d bytes\n", UNP.usize);
       close(i);
       free(UNP.outputbuf);
-      return CL_EIO;
+      return CL_EWRITE;
     }
     free(UNP.outputbuf);
-    if(cli_leavetemps_flag)
+    if(ctx->engine->keeptmp)
       cli_dbgmsg("autoit: file extracted to %s\n", tempfile);
     else 
       cli_dbgmsg("autoit: file successfully extracted\n");
     lseek(i, 0, SEEK_SET);
     if(cli_magic_scandesc(i, ctx) == CL_VIRUS) {
       close(i);
-      if(!cli_leavetemps_flag)
-        if(cli_unlink(tempfile)) return CL_EIO;
+      if(!ctx->engine->keeptmp)
+        if(cli_unlink(tempfile)) return CL_EUNLINK;
       return CL_VIRUS;
     }
     close(i);
-    if(!cli_leavetemps_flag) 
-      if (cli_unlink(tempfile)) return CL_EIO;
+    if(!ctx->engine->keeptmp) 
+      if (cli_unlink(tempfile)) return CL_EUNLINK;
   }
   return ret;
 }
@@ -868,29 +868,29 @@ static int ea06(int desc, cli_ctx *ctx, char *tmpd) {
     if((i = open(tempfile, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) {
       cli_dbgmsg("autoit: Can't create file %s\n", tempfile);
       free(buf);
-      return CL_EIO;
+      return CL_ECREAT;
     }
     if(cli_writen(i, buf, UNP.cur_output) != (int32_t)UNP.cur_output) {
       cli_dbgmsg("autoit: cannot write %d bytes\n", UNP.usize);
       close(i);
       free(buf);
-      return CL_EIO;
+      return CL_EWRITE;
     }
     free(buf);
-    if(cli_leavetemps_flag)
+    if(ctx->engine->keeptmp)
       cli_dbgmsg("autoit: %s extracted to %s\n", (script)?"script":"file", tempfile);
     else 
       cli_dbgmsg("autoit: %s successfully extracted\n", (script)?"script":"file");
     lseek(i, 0, SEEK_SET);
     if(cli_magic_scandesc(i, ctx) == CL_VIRUS) {
       close(i);
-      if(!cli_leavetemps_flag) 
-        if (cli_unlink(tempfile)) return CL_EIO;
+      if(!ctx->engine->keeptmp) 
+        if (cli_unlink(tempfile)) return CL_EUNLINK;
       return CL_VIRUS;
     }
     close(i);
-    if(!cli_leavetemps_flag)
-      if (cli_unlink(tempfile)) return CL_EIO;
+    if(!ctx->engine->keeptmp)
+      if (cli_unlink(tempfile)) return CL_EUNLINK;
   }
   return ret;
 }
@@ -908,18 +908,18 @@ int cli_scanautoit(int desc, cli_ctx *ctx, off_t offset) {
 
   lseek(desc, offset, SEEK_SET);
   if (cli_readn(desc, &version, 1)!=1)
-    return CL_EIO;
+    return CL_EREAD;
 
   cli_dbgmsg("in scanautoit()\n");
 
-  if (!(tmpd = cli_gentemp(NULL)))    
+  if (!(tmpd = cli_gentemp(ctx->engine->tmpdir)))    
     return CL_ETMPDIR;
   if (mkdir(tmpd, 0700)) {
     cli_dbgmsg("autoit: Can't create temporary directory %s\n", tmpd);
     free(tmpd);
     return CL_ETMPDIR;
   }
-  if (cli_leavetemps_flag)
+  if (ctx->engine->keeptmp)
     cli_dbgmsg("autoit: Extracting files to %s\n", tmpd);
 
   switch(version) {
@@ -940,7 +940,7 @@ int cli_scanautoit(int desc, cli_ctx *ctx, off_t offset) {
     r = CL_CLEAN;
   }
 
-  if (!cli_leavetemps_flag)
+  if (!ctx->engine->keeptmp)
     cli_rmdirs(tmpd);
 
   free(tmpd);
