@@ -53,10 +53,6 @@ static	char	const	rcsid[] = "$Id: blob.c,v 1.64 2007/02/12 22:25:14 njh Exp $";
 #include "scanners.h"
 #include "filetypes.h"
 
-#ifndef	CL_DEBUG
-#define	NDEBUG	/* map CLAMAV debug onto standard */
-#endif
-
 #ifndef	O_BINARY
 #define	O_BINARY	0
 #endif
@@ -411,7 +407,7 @@ fileblobCreate(void)
 int
 fileblobScanAndDestroy(fileblob *fb)
 {
-	if(cli_leavetemps_flag) {
+	if(fb->ctx && fb->ctx->engine->keeptmp) {
 		/* Can't remove the file, the caller must scan */
 		fileblobDestroy(fb);
 		return CL_CLEAN;
@@ -509,11 +505,11 @@ fileblobPartialSet(fileblob *fb, const char *fullname, const char *arg)
 	fb->fp = fdopen(fb->fd, "wb");
 
 	if(fb->fp == NULL) {
-		cli_errmsg("fileblobSetFilename: fdopen failed (%s)\n", strerror(errno));
+		cli_errmsg("fileblobSetFilename: fdopen failed\n");
 		close(fb->fd);
 		return;
 	}
-	blobSetFilename(&fb->b, NULL, fullname);
+	blobSetFilename(&fb->b, fb->ctx ? fb->ctx->engine->tmpdir : NULL, fullname);
 	if(fb->b.data)
 		if(fileblobAddData(fb, fb->b.data, fb->b.len) == 0) {
 			free(fb->b.data);
@@ -552,7 +548,7 @@ fileblobSetFilename(fileblob *fb, const char *dir, const char *filename)
 	fb->fp = fdopen(fb->fd, "wb");
 
 	if(fb->fp == NULL) {
-		cli_errmsg("fileblobSetFilename: fdopen failed (%s)\n", strerror(errno));
+		cli_errmsg("fileblobSetFilename: fdopen failed\n");
 		close(fb->fd);
 		free(fullname);
 		return;
@@ -594,7 +590,7 @@ fileblobAddData(fileblob *fb, const unsigned char *data, size_t len)
 					*ctx->scanned += (unsigned long)len / CL_COUNT_PRECISION;
 				fb->bytes_scanned += (unsigned long)len;
 				
-				if((len > 5) && cli_updatelimits(ctx, len)==CL_CLEAN && (cli_scanbuff(data, (unsigned int)len, ctx->virname, ctx->engine, CL_TYPE_BINARY_DATA) == CL_VIRUS)) {
+				if((len > 5) && cli_updatelimits(ctx, len)==CL_CLEAN && (cli_scanbuff(data, (unsigned int)len, 0, ctx->virname, ctx->engine, CL_TYPE_BINARY_DATA, NULL) == CL_VIRUS)) {
 					cli_dbgmsg("fileblobAddData: found %s\n", *ctx->virname);
 					fb->isInfected = 1;
 				}
@@ -603,8 +599,8 @@ fileblobAddData(fileblob *fb, const unsigned char *data, size_t len)
 #endif
 
 		if(fwrite(data, len, 1, fb->fp) != 1) {
-			cli_errmsg("fileblobAddData: Can't write %lu bytes to temporary file %s: %s\n",
-				(unsigned long)len, fb->b.name, strerror(errno));
+			cli_errmsg("fileblobAddData: Can't write %lu bytes to temporary file %s\n",
+				(unsigned long)len, fb->b.name);
 			return -1;
 		}
 		fb->isNotEmpty = 1;
