@@ -39,6 +39,7 @@ typedef enum {
     /* Please do not add any new types above this line */
     CL_TYPE_ERROR,
     CL_TYPE_MSEXE,
+    CL_TYPE_PE_DISASM,
     CL_TYPE_ELF,
     CL_TYPE_POSIX_TAR,
     CL_TYPE_OLD_TAR,
@@ -94,7 +95,7 @@ struct cli_matched_type {
 };
 
 cli_file_t cli_ftcode(const char *name);
-void cli_ftfree(struct cli_ftype *ftypes);
+void cli_ftfree(const struct cl_engine *engine);
 cli_file_t cli_filetype(const unsigned char *buf, size_t buflen, const struct cl_engine *engine);
 cli_file_t cli_filetype2(int desc, const struct cl_engine *engine);
 int cli_addtypesigs(struct cl_engine *engine);
