@@ -23,11 +23,6 @@
 #include "clamav-config.h"
 #endif
 
-/* assert() only enabled with ./configure --enable-debug */
-#ifndef CL_DEBUG
-#define NDEBUG
-#endif
-
 #include <stdio.h>
 
 #ifdef HAVE_UNISTD_H
@@ -77,12 +72,10 @@ typedef struct scanner {
 typedef int YY_BUFFER_STATE;
 
 static int yylex( YYSTYPE *lvalp, yyscan_t  );
-static void yy_delete_buffer( YY_BUFFER_STATE, yyscan_t);
 static YY_BUFFER_STATE yy_scan_bytes( const char *, size_t, yyscan_t scanner );
 static const char *yyget_text ( yyscan_t scanner );
 static int yyget_leng ( yyscan_t scanner );
 static int yylex_init ( yyscan_t * ptr_yy_globals ) ;
-static void yyset_debug (int debug_flag ,yyscan_t yyscanner );
 static int yylex_destroy ( yyscan_t yyscanner ) ;
 /* ----------- tokenizer end ---------------- */
 
@@ -115,12 +108,12 @@ struct tokens {
 struct parser_state {
 	unsigned long     var_uniq;
 	unsigned long     syntax_errors;
-	unsigned int      rec;
 	struct scope *global;
 	struct scope *current;
 	struct scope *list;
 	yyscan_t scanner;
 	struct tokens tokens;
+	unsigned int      rec;
 };
 
 static struct scope* scope_new(struct parser_state *state)
@@ -322,7 +315,7 @@ static inline int buf_outc(char c, struct buf *buf)
 {
 	if(buf->pos >= sizeof(buf->buf)) {
 		if(write(buf->outfd, buf->buf, sizeof(buf->buf)) != sizeof(buf->buf))
-			return CL_EIO;
+			return CL_EWRITE;
 		buf->pos = 0;
 	}
 	buf->buf[buf->pos++] = c;
@@ -337,11 +330,15 @@ static inline int buf_outs(const char *s, struct buf *buf)
 	i = buf->pos;
 	while(*s) {
 		while(i < buf_len && *s) {
-			buf->buf[i++] = tolower((unsigned char)(*s++));
+			if(isspace(*s & 0xff))
+				buf->buf[i++] = ' ';
+			else
+				buf->buf[i++] = tolower((unsigned char)(*s));
+			++s;
 		}
 		if(i == buf_len) {
 			if(write(buf->outfd, buf->buf, buf_len) < 0)
-				return CL_EIO;
+				return CL_EWRITE;
 		       i = 0;
 		}
 	}
@@ -473,7 +470,7 @@ static void scope_free_all(struct scope *p)
 	} while(p);
 }
 
-void cli_strtokenize(char *buffer, const char delim, const size_t token_count, const char **tokens);
+size_t cli_strtokenize(char *buffer, const char delim, const size_t token_count, const char **tokens);
 static int match_parameters(const yystype *tokens, const char ** param_names, size_t count)
 {
 	size_t i,j=0;
@@ -688,7 +685,7 @@ static void handle_de(yystype *tokens, size_t start, const size_t cnt, const cha
 	}
 }
 
-static int handle_unescape(struct tokens *tokens, size_t start, const size_t cnt)
+static int handle_unescape(struct tokens *tokens, size_t start)
 {
 	if(tokens->data[start].type == TOK_StringLiteral) {
 		char *R;
@@ -708,7 +705,7 @@ static int handle_unescape(struct tokens *tokens, size_t start, const size_t cnt
 
 
 /* scriptasylum dot com's JS encoder */
-static void handle_df(const yystype *tokens, size_t start, const size_t cnt, struct decode_result *res)
+static void handle_df(const yystype *tokens, size_t start, struct decode_result *res)
 {
 	char *str, *s1;
 	size_t len, s1_len, i;
@@ -761,7 +758,7 @@ static void run_folders(struct tokens *tokens)
 		    cstring &&
 		    !strcmp("unescape", cstring) && tokens->data[i+1].type == TOK_PAR_OPEN) {
 
-		  handle_unescape(tokens, i+2, tokens->cnt);
+		  handle_unescape(tokens, i+2);
 	  }
   }
 }
@@ -814,7 +811,7 @@ static void run_decoders(struct parser_state *state)
 		  /* TODO: also match signature of dF function (possibly
 		   * declared using unescape */
 
-		  handle_df(tokens->data, i+2, tokens->cnt, &res);
+		  handle_df(tokens->data, i+2, &res);
 	  } else if(i+2 < tokens->cnt && tokens->data[i].type == TOK_IDENTIFIER_NAME &&
 			  cstring &&
 			  !strcmp("eval", cstring) && tokens->data[i+1].type == TOK_PAR_OPEN) {
@@ -828,7 +825,7 @@ static void run_decoders(struct parser_state *state)
 		/* initialize embedded context */
 		memset(&state->tokens, 0, sizeof(state->tokens));
 		if(++state->rec > 16)
-			cli_dbgmsg(MODULE "recursion limit reached");
+			cli_dbgmsg(MODULE "recursion limit reached\n");
 		else {
 			cli_js_process_buffer(state, res.txtbuf.data, res.txtbuf.pos);
 			--state->rec;
@@ -853,7 +850,43 @@ static void run_decoders(struct parser_state *state)
 
 void cli_js_parse_done(struct parser_state* state)
 {
+	struct tokens * tokens = &state->tokens;
+	size_t par_balance = 0, i;
+	char end = '\0';
+	YYSTYPE val;
+
 	cli_dbgmsg(MODULE "in cli_js_parse_done()\n");
+	/* close unfinished token */
+	switch (state->scanner->state) {
+		case DoubleQString:
+			end = '"';
+			break;
+		case SingleQString:
+			end = '\'';
+			break;
+		default: /* make gcc happy */
+			break;
+	}
+	if (end != '\0')
+		cli_js_process_buffer(state, &end, 1);
+	/* close remaining paranthesis */
+	for (i=0;i<tokens->cnt;i++) {
+		if (tokens->data[i].type == TOK_PAR_OPEN)
+			par_balance++;
+		else if (tokens->data[i].type == TOK_PAR_CLOSE && par_balance > 0)
+			par_balance--;
+	}
+	if (par_balance > 0) {
+		memset(&val, 0, sizeof(val));
+		val.type = TOK_PAR_CLOSE;
+		TOKEN_SET(&val, cstring, ")");
+		while (par_balance-- > 0) {
+			add_token(state, &val);
+		}
+	}
+
+	/* we had to close unfinished strings, paranthesis,
+	 * so that the folders/decoders can run properly */
 	run_folders(&state->tokens);
 	run_decoders(state);
 
@@ -882,13 +915,17 @@ void cli_js_output(struct parser_state *state, const char *tempdir)
 		/* separate multiple scripts with \n */
 		buf_outc('\n', &buf);
 	}
+	buf_outs("<script>", &buf);
 	state->current = state->global;
 	for(i = 0; i < state->tokens.cnt; i++) {
 		if(state_update_scope(state, &state->tokens.data[i]))
 			lastchar = output_token(&state->tokens.data[i], state->current, &buf, lastchar);
 	}
+	/* add /script if not already there */
+	if(buf.pos < 9 || memcmp(buf.buf + buf.pos - 9, "</script>", 9))
+		buf_outs("</script>", &buf);
 	if(write(buf.outfd, buf.buf, buf.pos) < 0) {
-		cli_dbgmsg(MODULE "I/O error");
+		cli_dbgmsg(MODULE "I/O error\n");
 	}
 	close(buf.outfd);
 	cli_dbgmsg(MODULE "dumped/appended normalized script to: %s\n",filename);
@@ -928,7 +965,7 @@ void cli_js_process_buffer(struct parser_state *state, const char *buf, size_t n
 	if(!state->global) {
 		/* this state has either not been initialized,
 		 * or cli_js_parse_done() was already called on it */
-		cli_warnmsg(MODULE "invalid state");
+		cli_warnmsg(MODULE "invalid state\n");
 		return;
 	}
 	yyb = yy_scan_bytes(buf, n, state->scanner);
@@ -1111,7 +1148,6 @@ void cli_js_process_buffer(struct parser_state *state, const char *buf, size_t n
 		memset(&val, 0, sizeof(val));
 		val.vtype = vtype_undefined;
 	}
-	yy_delete_buffer(yyb, state->scanner);
 }
 
 struct parser_state *cli_js_init(void)
@@ -1130,7 +1166,6 @@ struct parser_state *cli_js_init(void)
 		free(state);
 		return NULL;
 	}
-	yyset_debug(1, state->scanner);
 	cli_dbgmsg(MODULE "cli_js_init() done\n");
 	return state;
 }
@@ -1401,14 +1436,6 @@ static int yy_scan_bytes(const char *p, size_t len, yyscan_t scanner)
 	return 0;
 }
 
-static void yyset_debug (int debug_flag ,yyscan_t yyscanner )
-{
-}
-
-static void yy_delete_buffer( YY_BUFFER_STATE yyb, yyscan_t scanner)
-{
-}
-
 static const char *yyget_text(yyscan_t scanner)
 {
 	assert(scanner->buf.data || scanner->yytext);
@@ -1512,12 +1539,16 @@ static int yylex(YYSTYPE *lvalp, yyscan_t  scanner)
 				return parseNumber(lvalp, scanner);
 			case SinglelineComment:
 				while(scanner->pos < scanner->insize) {
-					if(in[scanner->pos] == '\n')
+					/* htmlnorm converts \n to space, so
+					 * stop on space too */
+					if(in[scanner->pos] == '\n' || in[scanner->pos] == ' ')
 						break;
 					scanner->pos++;
 				}
 				scanner->state = Initial;
 				break;
+			default:
+				assert(0 && "Not reached");
 		}
 	}
 	return 0;
