@@ -1,7 +1,9 @@
 /*
  *  Extract component parts of ARJ archives.
  *
- *  Copyright (C) 2007 trog@uncon.org
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *
+ *  Authors: Trog
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -146,11 +148,10 @@ typedef struct arj_file_hdr_tag {
 #endif
 
 typedef struct arj_decode_tag {
-	int fd;
 	unsigned char *text;
+	int fd;
 	uint16_t blocksize;
 	uint16_t bit_buf;
-	unsigned char sub_bit_buf;
 	int bit_count;
 	uint32_t comp_size;
 	int16_t getlen, getbuf;
@@ -159,7 +160,9 @@ typedef struct arj_decode_tag {
 	unsigned char c_len[NC];
 	uint16_t c_table[CTABLESIZE];
 	unsigned char pt_len[NPT];
+	unsigned char sub_bit_buf;
 	uint16_t pt_table[PTABLESIZE];
+	int status;
 } arj_decode_t;
 
 static int fill_buf(arj_decode_t *decode_data, int n)
@@ -170,7 +173,11 @@ static int fill_buf(arj_decode_t *decode_data, int n)
 		if (decode_data->comp_size != 0) {
 			decode_data->comp_size--;
 			if (cli_readn(decode_data->fd, &decode_data->sub_bit_buf, 1) != 1) {
-				return CL_EIO;
+				/* the file is most likely corrupted, so
+				 * we return CL_EFORMAT instead of CL_EREAD
+				 */
+				decode_data->status = CL_EFORMAT;
+				return CL_EFORMAT;
 			}
 		} else {
 			decode_data->sub_bit_buf = 0;
@@ -210,7 +217,7 @@ static int write_text(int ofd, unsigned char *data, int length)
 	
 	count = cli_writen(ofd, data, length);
 	if (count != length) {
-		return CL_EIO;
+		return CL_EWRITE;
 	} else {
 		return CL_SUCCESS;
 	}
@@ -228,7 +235,8 @@ static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitle
 	for (i = 0; (int)i < nchar; i++) {
 		if (bitlen[i] >= 17) {
 			cli_dbgmsg("UNARJ: bounds exceeded\n");
-			return CL_EARJ;
+			decode_data->status = CL_EUNPACK;
+			return CL_EUNPACK;
 		}
 		count[bitlen[i]]++;
 	}
@@ -238,13 +246,15 @@ static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitle
 		start[i+1] = start[i] + (count[i] << (16 - i));
 	}
 	if (start[17] != (unsigned short) (1 << 16)) {
-		return CL_EARJ;
+		decode_data->status = CL_EUNPACK;
+		return CL_EUNPACK;
 	}
 	
 	jutbits = 16 - tablebits;
 	if (tablebits >= 17) {
 		cli_dbgmsg("UNARJ: bounds exceeded\n");
-		return CL_EARJ;
+		decode_data->status = CL_EUNPACK;
+		return CL_EUNPACK;
 	}
 	for (i = 1; (int)i <= tablebits; i++) {
 		start[i] >>= jutbits;
@@ -261,7 +271,8 @@ static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitle
 		while (i != k) {
 			if (i >= tablesize) {
 				cli_dbgmsg("UNARJ: bounds exceeded\n");
-				return CL_EARJ;
+				decode_data->status = CL_EUNPACK;
+				return CL_EUNPACK;
 			}
 			table[i++] = 0;
 		}
@@ -275,13 +286,15 @@ static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitle
 		}
 		if (len >= 17) {
 			cli_dbgmsg("UNARJ: bounds exceeded\n");
-			return CL_EARJ;
+			decode_data->status = CL_EUNPACK;
+			return CL_EUNPACK;
 		}
 		k = start[len];
 		nextcode = k + weight[len];
 		if ((int)len <= tablebits) {
 			if (nextcode > (unsigned int) tablesize) {
-				return CL_EARJ;
+				decode_data->status = CL_EUNPACK;
+				return CL_EUNPACK;
 			}
 			for (i = start[len]; i < nextcode; i++) {
 				table[i] = ch;
@@ -293,14 +306,16 @@ static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitle
 				if (*p == 0) {
 					if (avail >= (2 * NC - 1)) {
 						cli_dbgmsg("UNARJ: bounds exceeded\n");
-						return CL_EARJ;
+						decode_data->status = CL_EUNPACK;
+						return CL_EUNPACK;
 					}
 					decode_data->right[avail] = decode_data->left[avail] = 0;
 					*p = avail++;
 				}
 				if (*p >= (2 * NC - 1)) {
 					cli_dbgmsg("UNARJ: bounds exceeded\n");
-					return CL_EARJ;
+					decode_data->status = CL_EUNPACK;
+					return CL_EUNPACK;
 				}
 				if (k & mask) {
 					p = &decode_data->right[*p];
@@ -317,7 +332,7 @@ static int make_table(arj_decode_t *decode_data, int nchar, unsigned char *bitle
 	return CL_SUCCESS;
 }
 
-static void read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_special)
+static int read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_special)
 {
 	int i, n;
 	short c;
@@ -327,7 +342,8 @@ static void read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_speci
 	if (n == 0) {
 		if (nn > NPT) {
 			cli_dbgmsg("UNARJ: bounds exceeded\n");
-			return;
+			decode_data->status = CL_EUNPACK;
+			return CL_EUNPACK;
 		}
 		c = arj_getbits(decode_data, nbit);
 		for (i = 0; i < nn; i++) {
@@ -348,9 +364,15 @@ static void read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_speci
 				}
 			}
 			fill_buf(decode_data, (c < 7) ? 3 : (int)(c - 3));
+			if (decode_data->status != CL_SUCCESS) {
+				return decode_data->status;
+			}
 			decode_data->pt_len[i++] = (unsigned char) c;
 			if (i == i_special) {
 				c = arj_getbits(decode_data, 2);
+				if (decode_data->status != CL_SUCCESS) {
+					return decode_data->status;
+				}
 				while ((--c >= 0) && (i < NPT)) {
 					decode_data->pt_len[i++] = 0;
 				}
@@ -359,8 +381,11 @@ static void read_pt_len(arj_decode_t *decode_data, int nn, int nbit, int i_speci
 		while ((i < nn) && (i < NPT)) {
 			decode_data->pt_len[i++] = 0;
 		}
-		make_table(decode_data, nn, decode_data->pt_len, 8, decode_data->pt_table, PTABLESIZE);
+		if (make_table(decode_data, nn, decode_data->pt_len, 8, decode_data->pt_table, PTABLESIZE) != CL_SUCCESS) {
+			return CL_EUNPACK;
+		}
 	}
+	return CL_SUCCESS;
 }
 
 static int read_c_len(arj_decode_t *decode_data)
@@ -369,8 +394,14 @@ static int read_c_len(arj_decode_t *decode_data)
 	unsigned short mask;
 	
 	n = arj_getbits(decode_data, CBIT);
+	if (decode_data->status != CL_SUCCESS) {
+		return decode_data->status;
+	}
 	if (n == 0) {
 		c = arj_getbits(decode_data, CBIT);
+		if (decode_data->status != CL_SUCCESS) {
+			return decode_data->status;
+		}
 		for (i = 0; i < NC; i++) {
 			decode_data->c_len[i] = 0;
 		}
@@ -386,6 +417,7 @@ static int read_c_len(arj_decode_t *decode_data)
 				do {
 					if (c >= (2 * NC - 1)) {
 						cli_warnmsg("ERROR: bounds exceeded\n");
+						decode_data->status = CL_EFORMAT;
 						return CL_EFORMAT;
 					}
 					if (decode_data->bit_buf & mask) {
@@ -398,9 +430,13 @@ static int read_c_len(arj_decode_t *decode_data)
 			}
 			if (c >= 19) {
 				cli_dbgmsg("UNARJ: bounds exceeded\n");
-				return CL_EARJ;
+				decode_data->status = CL_EUNPACK;
+				return CL_EUNPACK;
 			}
 			fill_buf(decode_data, (int)(decode_data->pt_len[c]));
+			if (decode_data->status != CL_SUCCESS) {
+				return decode_data->status;
+			}	
 			if (c <= 2) {
 				if (c == 0) {
 					c = 1;
@@ -409,9 +445,13 @@ static int read_c_len(arj_decode_t *decode_data)
 				} else {
 					c = arj_getbits(decode_data, CBIT) + 20;
 				}
+				if (decode_data->status != CL_SUCCESS) {
+					return decode_data->status;
+				}		
 				while (--c >= 0) {
 					if (i >= NC) {
 						cli_warnmsg("ERROR: bounds exceeded\n");
+						decode_data->status = CL_EFORMAT;
 						return CL_EFORMAT;
 					}
 					decode_data->c_len[i++] = 0;
@@ -419,6 +459,7 @@ static int read_c_len(arj_decode_t *decode_data)
 			} else {
 				if (i >= NC) {
 					cli_warnmsg("ERROR: bounds exceeded\n");
+					decode_data->status = CL_EFORMAT;
 					return CL_EFORMAT;
 				}
 				decode_data->c_len[i++] = (unsigned char) (c - 2);
@@ -427,7 +468,9 @@ static int read_c_len(arj_decode_t *decode_data)
 		while (i < NC) {
 			decode_data->c_len[i++] = 0;
 		}
-		make_table(decode_data, NC, decode_data->c_len, 12, decode_data->c_table, CTABLESIZE);
+		if (make_table(decode_data, NC, decode_data->c_len, 12, decode_data->c_table, CTABLESIZE) != CL_SUCCESS) {
+			return CL_EUNPACK;
+		}
 	}
 	return CL_SUCCESS;
 }
@@ -450,6 +493,7 @@ static uint16_t decode_c(arj_decode_t *decode_data)
 		do {
 			if (j >= (2 * NC - 1)) {
 				cli_warnmsg("ERROR: bounds exceeded\n");
+				decode_data->status = CL_EUNPACK;
 				return 0;
 			}
 			if (decode_data->bit_buf & mask) {
@@ -474,6 +518,7 @@ static uint16_t decode_p(arj_decode_t *decode_data)
 		do {
 			if (j >= (2 * NC - 1)) {
 				cli_warnmsg("ERROR: bounds exceeded\n");
+				decode_data->status = CL_EUNPACK;
 				return 0;
 			}
 			if (decode_data->bit_buf & mask) {
@@ -508,8 +553,10 @@ static int decode(int fd, arj_metadata_t *metadata)
 	decode_data.comp_size = metadata->comp_size;
 	ret = decode_start(&decode_data);
 	if (ret != CL_SUCCESS) {
+		free(decode_data.text);
 		return ret;
 	}
+	decode_data.status = CL_SUCCESS;
 
 	while (count < metadata->orig_size) {
 		if ((chr = decode_c(&decode_data)) <= UCHAR_MAX) {
@@ -517,7 +564,10 @@ static int decode(int fd, arj_metadata_t *metadata)
 			count++;
 			if (++out_ptr >= DDICSIZ) {
 				out_ptr = 0;
-				write_text(metadata->ofd, decode_data.text, DDICSIZ);
+				if (write_text(metadata->ofd, decode_data.text, DDICSIZ) != CL_SUCCESS) {
+					free(decode_data.text);
+					return CL_EWRITE;
+				}
 			}
 		} else {
 			j = chr - (UCHAR_MAX + 1 - THRESHOLD);
@@ -539,7 +589,10 @@ static int decode(int fd, arj_metadata_t *metadata)
 					decode_data.text[out_ptr] = decode_data.text[i];
 					if (++out_ptr >= DDICSIZ) {
 						out_ptr = 0;
-						write_text(metadata->ofd, decode_data.text, DDICSIZ);
+						if (write_text(metadata->ofd, decode_data.text, DDICSIZ) != CL_SUCCESS) {
+							free(decode_data.text);
+							return CL_EWRITE;
+						}
 					}
 					if (++i >= DDICSIZ) {
 						i = 0;
@@ -547,6 +600,10 @@ static int decode(int fd, arj_metadata_t *metadata)
 				}
 			}
 		}
+		if (decode_data.status != CL_SUCCESS) {
+			free(decode_data.text);
+			return decode_data.status;
+		}
 	}
 	if (out_ptr != 0) {
 		write_text(metadata->ofd, decode_data.text, out_ptr);
@@ -623,21 +680,37 @@ static int decode_f(int fd, arj_metadata_t *metadata)
 		return ret;
 	}
     	decode_data.getlen = decode_data.getbuf = 0;
-
+	decode_data.status = CL_SUCCESS;
+	
 	while (count < metadata->orig_size) {
 		chr = decode_len(&decode_data);
+		if (decode_data.status != CL_SUCCESS) {
+			free(decode_data.text);
+			return decode_data.status;
+		}		
 		if (chr == 0) {
 			ARJ_GETBITS(dd, chr, CHAR_BIT);
+			if (decode_data.status != CL_SUCCESS) {
+				free(decode_data.text);
+				return decode_data.status;
+			}
 			decode_data.text[out_ptr] = (unsigned char) chr;
 			count++;
 			if (++out_ptr >= DDICSIZ) {
 				out_ptr = 0;
-				write_text(metadata->ofd, decode_data.text, DDICSIZ);
+				if (write_text(metadata->ofd, decode_data.text, DDICSIZ) != CL_SUCCESS) {
+					free(decode_data.text);
+					return CL_EWRITE;
+				}
 			}
 		} else {
 			j = chr - 1 + THRESHOLD;
 			count += j;
 			pos = decode_ptr(&decode_data);
+			if (decode_data.status != CL_SUCCESS) {
+				free(decode_data.text);
+				return decode_data.status;
+			}
 			if ((i = out_ptr - pos - 1) < 0) {
 				i += DDICSIZ;
 			}
@@ -649,7 +722,10 @@ static int decode_f(int fd, arj_metadata_t *metadata)
 				decode_data.text[out_ptr] = decode_data.text[i];
 				if (++out_ptr >= DDICSIZ) {
 					out_ptr = 0;
-					write_text(metadata->ofd, decode_data.text, DDICSIZ);
+					if (write_text(metadata->ofd, decode_data.text, DDICSIZ) != CL_SUCCESS) {
+						free(decode_data.text);
+						return CL_EWRITE;
+					}
 				}
 				if (++i >= DDICSIZ) {
 					i = 0;
@@ -987,7 +1063,7 @@ int cli_unarj_extract_file(int fd, const char *dirname, arj_metadata_t *metadata
 		offset = lseek(fd, 0, SEEK_CUR) + metadata->comp_size;
 		cli_dbgmsg("Target offset: %ld\n", offset);
 		if (lseek(fd, offset, SEEK_SET) != offset) {
-			return CL_EARJ;
+			return CL_ESEEK;
 		}
 		return CL_SUCCESS;
 	}
@@ -1002,7 +1078,7 @@ int cli_unarj_extract_file(int fd, const char *dirname, arj_metadata_t *metadata
 		case 0:
 			ret = arj_unstore(fd, metadata->ofd, metadata->comp_size);
 			if (ret != metadata->comp_size) {
-				ret = CL_EIO;
+				ret = CL_EWRITE;
 			} else {
 				ret = CL_SUCCESS;
 			}
@@ -1010,10 +1086,10 @@ int cli_unarj_extract_file(int fd, const char *dirname, arj_metadata_t *metadata
 		case 1:
 		case 2:
 		case 3:
-			decode(fd, metadata);
+			ret = decode(fd, metadata);
 			break;
 		case 4:
-			decode_f(fd, metadata);
+			ret = decode_f(fd, metadata);
 			break;
 		default:
 			ret = CL_EFORMAT;
