@@ -1084,7 +1084,7 @@ int sigtool_vba_scandir (const char *dirname, int hex_output, struct uniq *U)
 	    vbaname[sizeof(vbaname)-1] = '\0';
 	    fd = open(vbaname, O_RDONLY|O_BINARY);
 	    if (fd == -1) continue;
-	    if ((fullname = cli_ppt_vba_read(fd))) {
+	    if ((fullname = cli_ppt_vba_read(fd, NULL))) {
 	      sigtool_scandir(fullname, hex_output);
 	      cli_rmdirs(fullname);
 	      free(fullname);
@@ -1107,7 +1107,8 @@ int sigtool_vba_scandir (const char *dirname, int hex_output, struct uniq *U)
 	    }
 
 	    for (i = 0; i < vba_project->count; i++) {
-		data = (unsigned char *)cli_wm_decrypt_macro(fd, vba_project->offset[i], vba_project->length[i], vba_project->key[i]);
+		data_len = vba_project->length[i];
+		data = (unsigned char *)cli_wm_decrypt_macro(fd, vba_project->offset[i], data_len , vba_project->key[i]);
 		if(data) {
 		    data = (unsigned char *) realloc (data, data_len + 1);
 		    data[data_len]='\0';
